@@ -5,7 +5,7 @@
 /*    Mac FOND support.  Written by just@letterror.com.                    */
 /*  Heavily Fixed by mpsuzuki, George Williams and Sean McBride            */
 /*                                                                         */
-/*  Copyright 1996-2001, 2002, 2003, 2004, 2005, 2006 by                   */
+/*  Copyright 1996-2001, 2002, 2003, 2004, 2005, 2006, 2007 by             */
 /*  Just van Rossum, David Turner, Robert Wilhelm, and Werner Lemberg.     */
 /*                                                                         */
 /*  This file is part of the FreeType project, and may only be used,       */
@@ -70,8 +70,10 @@
   /* This is for Mac OS X.  Without redefinition, OS_INLINE */
   /* expands to `static inline' which doesn't survive the   */
   /* -ansi compilation flag of GCC.                         */
+#if !HAVE_ANSI_OS_INLINE
 #undef  OS_INLINE
 #define OS_INLINE   static __inline__
+#endif
 #include <Carbon/Carbon.h>
 #else
 #include <Resources.h>
@@ -86,6 +88,8 @@
 #include <FSp_fopen.h>
 #endif
 
+#define FT_DEPRECATED_ATTRIBUTE
+
 #include FT_MAC_H
 
 #undef FT_GetFile_From_Mac_Name
@@ -149,6 +153,10 @@
                             FSSpec*      pathSpec,
                             FT_Long*     face_index )
   {
+    FT_UNUSED( fontName );
+    FT_UNUSED( pathSpec );
+    FT_UNUSED( face_index );
+
     return FT_Err_Unimplemented_Feature;
   }
 
@@ -241,22 +249,33 @@
 #endif /* HAVE_QUICKDRAW_CARBON */
 
 
-#if !HAVE_ATS
+#if HAVE_ATS
 
-  FT_EXPORT_DEF( FT_Error )
-  FT_GetFile_From_Mac_ATS_Name( const char*  fontName,
-                                FSSpec*      pathSpec,
-                                FT_Long*     face_index )
+  /* Private function.                                         */
+  /* The FSSpec type has been discouraged for a long time,     */
+  /* but for some reason, there is no FSRef version of         */
+  /* ATSFontGetFileSpecification(), so we made our own.        */
+  /* Apple will provide one eventually.                        */
+  static OSStatus
+  FT_ATSFontGetFileReference( ATSFontRef  ats_font_id,
+                              FSRef*      ats_font_ref )
   {
-    return FT_Err_Unimplemented_Feature;
+    OSStatus  err;
+    FSSpec    spec;
+
+
+    err = ATSFontGetFileSpecification( ats_font_id, &spec );
+    if ( noErr == err )
+      err = FSpMakeFSRef( &spec, ats_font_ref );
+
+    return err;
   }
 
-#else
 
-  FT_EXPORT_DEF( FT_Error )
-  FT_GetFile_From_Mac_ATS_Name( const char*  fontName,
-                                FSSpec*      pathSpec,
-                                FT_Long*     face_index )
+  static FT_Error
+  FT_GetFileRef_From_Mac_ATS_Name( const char*  fontName,
+                                   FSRef*       ats_font_ref,
+                                   FT_Long*     face_index )
   {
     CFStringRef  cf_fontName;
     ATSFontRef   ats_font_id;
@@ -268,40 +287,119 @@
                                              kCFStringEncodingMacRoman );
     ats_font_id = ATSFontFindFromName( cf_fontName,
                                        kATSOptionFlagsUnRestrictedScope );
+    CFRelease( cf_fontName );
 
     if ( ats_font_id == 0 || ats_font_id == 0xFFFFFFFFUL )
       return FT_Err_Unknown_File_Format;
 
-    if ( 0 != ATSFontGetFileSpecification( ats_font_id, pathSpec ) )
+    if ( noErr != FT_ATSFontGetFileReference( ats_font_id, ats_font_ref ) )
       return FT_Err_Unknown_File_Format;
 
     /* face_index calculation by searching preceding fontIDs */
     /* with same FSRef                                       */
     {
-      int     i;
-      FSSpec  f;
+      ATSFontRef  id2 = ats_font_id - 1;
+      FSRef       ref2;
 
 
-      for ( i = 1; i < ats_font_id; i++ )
+      while ( id2 > 0 )
       {
-        if ( 0 != ATSFontGetFileSpecification( ats_font_id - i,
-                                               &f               ) ||
-             f.vRefNum != pathSpec->vRefNum                       ||
-             f.parID   != pathSpec->parID                         ||
-             f.name[0] != pathSpec->name[0]                       ||
-             0 != ft_strncmp( (char *)f.name + 1,
-                              (char *)pathSpec->name + 1,
-                              f.name[0]                           ) )
+        if ( noErr != FT_ATSFontGetFileReference( id2, &ref2 ) )
           break;
+        if ( noErr != FSCompareFSRefs( ats_font_ref, &ref2 ) )
+          break;
+
+        id2--;
       }
-      *face_index = ( i - 1 );
+      *face_index = ats_font_id - ( id2 + 1 );
     }
+
+    return FT_Err_Ok;
+  }
+
+#endif
+
+#if !HAVE_ATS
+
+  FT_EXPORT_DEF( FT_Error )
+  FT_GetFilePath_From_Mac_ATS_Name( const char*  fontName,
+                                    UInt8*       path,
+                                    UInt32       maxPathSize,
+                                    FT_Long*     face_index )
+  {
+    FT_UNUSED( fontName );
+    FT_UNUSED( path );
+    FT_UNUSED( maxPathSize );
+    FT_UNUSED( face_index );
+
+    return FT_Err_Unimplemented_Feature;
+  }
+
+#else
+
+  FT_EXPORT_DEF( FT_Error )
+  FT_GetFilePath_From_Mac_ATS_Name( const char*  fontName,
+                                    UInt8*       path,
+                                    UInt32       maxPathSize,
+                                    FT_Long*     face_index )
+  {
+    FSRef     ref;
+    FT_Error  err;
+
+
+    err = FT_GetFileRef_From_Mac_ATS_Name( fontName, &ref, face_index );
+    if ( FT_Err_Ok != err )
+      return err;
+
+    if ( noErr != FSRefMakePath( &ref, path, maxPathSize ) )
+      return FT_Err_Unknown_File_Format;
+
     return FT_Err_Ok;
   }
 
 #endif /* HAVE_ATS */
 
 
+#if !HAVE_FSSPEC
+
+  FT_EXPORT_DEF( FT_Error )
+  FT_GetFile_From_Mac_ATS_Name( const char*  fontName,
+                                FSSpec*      pathSpec,
+                                FT_Long*     face_index )
+  {
+    FT_UNUSED( fontName );
+    FT_UNUSED( pathSpec );
+    FT_UNUSED( face_index );
+
+    return FT_Err_Unimplemented_Feature;
+  }
+
+#else
+
+  /* This function is deprecated because FSSpec is deprecated in Mac OS X. */
+  FT_EXPORT_DEF( FT_Error )
+  FT_GetFile_From_Mac_ATS_Name( const char*  fontName,
+                                FSSpec*      pathSpec,
+                                FT_Long*     face_index )
+  {
+    FSRef     ref;
+    FT_Error  err;
+
+
+    err = FT_GetFileRef_From_Mac_ATS_Name( fontName, &ref, face_index );
+    if ( FT_Err_Ok != err )
+      return err;
+
+    if ( noErr != FSGetCatalogInfo( &ref, kFSCatInfoNone, NULL, NULL,
+                                    pathSpec, NULL ) )
+      return FT_Err_Unknown_File_Format;
+
+    return FT_Err_Ok;
+  }
+
+#endif
+
+
 #if defined( __MWERKS__ ) && !TARGET_RT_MAC_MACHO
 
 #define STREAM_FILE( stream )  ( (FT_FILE*)stream->descriptor.pointer )
@@ -357,7 +455,13 @@
 
     while ( 1 )
     {
-      q = p + FT_MIN( 255, ft_strlen( p ) );
+      int  len = ft_strlen( p );
+
+
+      if ( len > 255 )
+        len = 255;
+
+      q = p + len;
 
       if ( q == p )
         return 0;
@@ -495,7 +599,7 @@
 #else
 
     FSSpec  spec;
-    FInfo  finfo;
+    FInfo   finfo;
 
 
     if ( noErr != FT_FSPathMakeSpec( pathname, &spec, FALSE ) )
@@ -583,7 +687,6 @@
      to load a face OTHER than the first one in the FOND!
   */
 
-
   static void
   parse_fond( char*   fond_data,
               short*  have_sfnt,
@@ -688,10 +791,10 @@
 
 
   static  FT_Error
-  lookup_lwfn_by_fond( const UInt8*     path_fond,
-                       const StringPtr  base_lwfn,
-                       UInt8*           path_lwfn,
-                       int              path_size )
+  lookup_lwfn_by_fond( const UInt8*      path_fond,
+                       ConstStr255Param  base_lwfn,
+                       UInt8*            path_lwfn,
+                       int               path_size )
   {
 
 #if HAVE_FSREF
@@ -716,7 +819,7 @@
     if ( ft_strlen( (char *)path_lwfn ) + 1 + base_lwfn[0] > path_size )
       return FT_Err_Invalid_Argument;
 
-    /* now we have absolute dirname in lookup_path */
+    /* now we have absolute dirname in path_lwfn */
     if ( path_lwfn[0] == '/' )
       ft_strcat( (char *)path_lwfn, "/" );
     else
@@ -1360,6 +1463,11 @@
 
 #if !HAVE_FSREF
 
+    FT_UNUSED( library );
+    FT_UNUSED( ref );
+    FT_UNUSED( face_index );
+    FT_UNUSED( aface );
+
     return FT_Err_Unimplemented_Feature;
 
 #else
@@ -1443,6 +1551,11 @@
 
 #else
 
+    FT_UNUSED( library );
+    FT_UNUSED( spec );
+    FT_UNUSED( face_index );
+    FT_UNUSED( aface );
+
     return FT_Err_Unimplemented_Feature;
 
 #endif /* HAVE_FSREF, HAVE_FSSPEC */
